/*
 * _dm.h - Emulation of the Palm OS DataMgr API.
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#ifndef OS_PALM

#define noRecordSelected		unsigned( -1 )
#define noRecordSelectedID		unsigned( -1 )

#define dmRecordIDReservedRange		1		// The range of upper bits in the database's
							// uniqueIDSeed from 0 to this number are
							// reserved and not randomly picked when a
							// database is created.
#define dmDefaultRecordsID		0		// Records in a default database are copied
							// with their uniqueIDSeeds set in this range.
#define dmUnusedRecordID		0		// Record ID not allowed on the device.

#define dmModeReadOnly			0x0001		// read access
#define dmModeWrite			0x0002		// write access
#define dmModeReadWrite			0x0003		// read & write access
#define dmModeLeaveOpen			0x0004		// leave open when app quits
#define dmModeExclusive			0x0008		// don't let anyone else open it
#define dmModeShowSecret		0x0010		// force show of secret records

#define dmErrMemError			( dmErrorClass | 1 )
#define dmErrIndexOutOfRange		( dmErrorClass | 2 )
#define dmErrInvalidParam		( dmErrorClass | 3 )
#define dmErrReadOnly			( dmErrorClass | 4 )
#define dmErrDatabaseOpen		( dmErrorClass | 5 )
#define dmErrCantOpen			( dmErrorClass | 6 )
#define dmErrCantFind			( dmErrorClass | 7 )
#define dmErrRecordInWrongCard		( dmErrorClass | 8 )
#define dmErrCorruptDatabase		( dmErrorClass | 9 )
#define dmErrRecordDeleted		( dmErrorClass | 10 )
#define dmErrRecordArchived		( dmErrorClass | 11 )
#define dmErrNotRecordDB		( dmErrorClass | 12 )
#define dmErrNotResourceDB		( dmErrorClass | 13 )
#define dmErrROMBased			( dmErrorClass | 14 )
#define dmErrRecordBusy			( dmErrorClass | 15 )
#define dmErrResourceNotFound		( dmErrorClass | 16 )
#define dmErrNoOpenDatabase		( dmErrorClass | 17 )
#define dmErrInvalidCategory		( dmErrorClass | 18 )
#define dmErrNotValidRecord		( dmErrorClass | 19 )
#define dmErrWriteOutOfBounds		( dmErrorClass | 20 )
#define dmErrSeekFailed			( dmErrorClass | 21 )
#define dmErrAlreadyOpenForWrites	( dmErrorClass | 22 )
#define dmErrOpenedByAnotherTask	( dmErrorClass | 23 )
#define dmErrUniqueIDNotFound		( dmErrorClass | 24 )
#define dmErrAlreadyExists		( dmErrorClass | 25 )
#define dmErrInvalidDatabaseName	( dmErrorClass | 26 )
#define dmErrDatabaseProtected		( dmErrorClass | 27 )
#define dmErrDatabaseNotProtected	( dmErrorClass | 28 )

Err DmWrite( void* recordP, UInt32 offset, void const* srcP, UInt32 bytes );
Err DmStrCopy( void* recordP, UInt32 offset, Char const* srcP );
Err DmSet( void* recordP, UInt32 offset, UInt32 bytes, UInt8 value );

inline Err
DmOpenDatabaseInfo( DmOpenRef dbP, LocalID* dbIDP, 
		    UInt16* openCountP, UInt16* modeP, UInt16* cardNoP,
		    Boolean* resDBP )
{
	Assert( !dbIDP );
	Assert( !cardNoP );

	SUPPRESS_UNUSED_VAR( dbIDP );
	SUPPRESS_UNUSED_VAR( cardNoP );

	return dbP->DmOpenDatabaseInfo( openCountP, modeP, resDBP );
}

inline Err
DmDatabaseInfo( DmOpenRef dbP,
		Char* nameP, UInt16* attributesP,
		UInt16* versionP, UInt32* crDateP,
		UInt32* modDateP, UInt32* bckUpDateP,
		UInt32* modNumP, LocalID* appInfoIDP,
		LocalID* sortInfoIDP, UInt32* typeP,
		UInt32* creatorP )
{
	return dbP->DmDatabaseInfo( nameP, attributesP, versionP,
				    crDateP, modDateP, bckUpDateP,
				    modNumP, appInfoIDP, sortInfoIDP,
				    typeP, creatorP );
}

inline Err
DmSetDatabaseInfo( DmOpenRef dbP,
		   Char const* nameP, UInt16* attributesP,
		   UInt16* versionP, UInt32* crDateP,
		   UInt32* modDateP, UInt32* bckUpDateP,
		   UInt32* modNumP, LocalID* appInfoIDP,
		   LocalID* sortInfoIDP, UInt32* typeP,
		   UInt32* creatorP )
{
	return dbP->DmSetDatabaseInfo( nameP, attributesP, versionP,
				       crDateP, modDateP, bckUpDateP,
				       modNumP, appInfoIDP, sortInfoIDP,
				       typeP, creatorP );
}

inline Err
DmOpenDatabaseInfo( DmOpenRef dbP, UInt16* openCountP, UInt16* modeP, Boolean* resDBP )
{
	return dbP->DmOpenDatabaseInfo( openCountP, modeP, resDBP );
}

inline Err
DmCloseDatabase( DmOpenRef dbP )
{
	dbP->Close();
	return 0;
}

inline unsigned
DmNumRecords( DmOpenRef dbP )
{
	return dbP->DmNumRecords();
}

inline Err
DmRecordInfo( DmOpenRef dbP, unsigned index, UInt16* attrP, UInt32* uniqueIDP, LocalID* chunkIDP )
{
	return dbP->DmRecordInfo( index, attrP, uniqueIDP, chunkIDP );
}

inline Err
DmSetRecordInfo( DmOpenRef dbP, unsigned index, UInt16* attrP, UInt32* uniqueIDP )
{
	return dbP->DmSetRecordInfo( index, attrP, uniqueIDP );
}

inline Err
DmAttachRecord( DmOpenRef dbP, unsigned* atP, MemHandle newH, MemHandle* oldHP )
{
	return dbP->DmAttachRecord( atP, newH, oldHP );
}

inline Err
DmDetachRecord( DmOpenRef dbP, unsigned index, MemHandle* oldHP )
{
	return dbP->DmDetachRecord( index, oldHP );
}

inline Err
DmMoveRecord( DmOpenRef dbP, unsigned from, unsigned to )
{
	return dbP->DmMoveRecord( from, to );
}

inline MemHandle
DmNewRecord( DmOpenRef dbP, unsigned* atP, UInt32 size )
{
	return dbP->DmNewRecord( atP, size );
}

inline Err
DmRemoveRecord( DmOpenRef dbP, unsigned index )
{
	return dbP->DmRemoveRecord( index );
}

inline MemHandle
DmNewHandle( DmOpenRef dbP, UInt32 size )
{
	return dbP->DmNewHandle( size );
}

inline Err
DmFindRecordByID( DmOpenRef dbP, UInt32 uniqueID, unsigned* indexP )
{
	return dbP->DmFindRecordByID( uniqueID, indexP );
}

inline MemHandle
DmQueryRecord( DmOpenRef dbP, unsigned index )
{
	return dbP->DmQueryRecord( index );
}

inline MemHandle
DmGetRecord( DmOpenRef dbP, unsigned index )
{
	return dbP->DmGetRecord( index );
}

inline MemHandle
DmResizeRecord( DmOpenRef dbP, unsigned index, UInt32 newSize )
{
	return dbP->DmResizeRecord( index, newSize );
}

inline Err
DmReleaseRecord( DmOpenRef dbP, unsigned index, Boolean dirty )
{
	return dbP->DmReleaseRecord( index, dirty );
}

#if 0
inline Err
DmMoveCategory( DmOpenRef dbP, UInt16 toCategory, UInt16 fromCategory, Boolean dirty )
{
	return dbP->DmMoveCategory( toCategory, fromCategory, dirty );
}
#endif

inline Err
DmQuickSort( DmOpenRef dbP, DmComparF* compar, Int32 other )
{
	return dbP->DmQuickSort( compar, other );
}

inline unsigned
DmFindSortPosition( DmOpenRef dbP, void* newRecord, DmComparF* compar, Int32 other )
{
	return dbP->DmFindSortPosition( newRecord, compar, other );
}

#endif // !OS_PALM

