/*
 * _mem.h - MemoryMgr layer.
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#ifndef OS_PALM

#define memPtrLockCount			15

#define memErrChunkLocked		( memErrorClass | 1 )
#define memErrNotEnoughSpace		( memErrorClass | 2 )
#define memErrInvalidParam		( memErrorClass | 3 )	// Invalid param or requested size is too big.
#define memErrChunkNotLocked		( memErrorClass | 4 )
#define memErrCardNotPresent		( memErrorClass | 5 )
#define memErrNoCardHeader		( memErrorClass | 6 )
#define memErrInvalidStoreHeader	( memErrorClass | 7 )
#define memErrRAMOnlyCard		( memErrorClass | 8 )
#define memErrWriteProtect		( memErrorClass | 9 )
#define memErrNoRAMOnCard		( memErrorClass | 10 )
#define memErrNoStore			( memErrorClass | 11 )
#define memErrROMOnlyCard		( memErrorClass | 12 )

MemPtr		MemChunkNew( UInt16 heapID, UInt32 size, UInt16 attr );
Err		MemChunkFree( MemPtr chunkDataP );
MemPtr		MemPtrNew( UInt32 size );
#define		MemPtrFree( p )		MemChunkFree( p )
Err		MemPtrUnlock( MemPtr p );
#define		MemPtrUnlockRaw( p )	MemPtrUnlock( p )

inline Err	MemPtrUnlock( void const* p ) { return MemPtrUnlock( const_cast<void*>( p ) ); }

MemHandle	MemHandleNew( UInt32 size );
Err		MemHandleFree( MemHandle h );

MemHandle	MemPtrRecoverHandle( MemPtr p );
UInt32		MemPtrSize( MemPtr p );

UInt32		MemHandleSize( MemHandle h );
UInt16		MemHandleLockCount( MemHandle h );
LocalID		MemHandleToLocalID( MemHandle h );

Err		MemHandleResize( MemHandle h, UInt32 newSize );
MemPtr		MemHandleLock( MemHandle h );
Err		MemHandleUnlock( MemHandle h );
Err		MemHandleResetLock( MemHandle h );

MemPtr		MemLocalIDToGlobal( LocalID local, UInt16 cardNo );
MemPtr		MemLocalIDToLockedPtr( LocalID local, UInt16 cardNo );

#if 0
MemPtr		MemEmergencyPtrFromHandle( MemHandle h );
#endif

inline Err
MemMove( void* dstP, void const* sP, Int32 numBytes )
{
	memmove( dstP, sP, numBytes );
	return 0;
}

inline Err
MemSet( void* dstP, Int32 numBytes, UInt8 value )
{
	memset( dstP, value, numBytes );
	return 0;
}

inline int
MemCmp( void const* s1, void const* s2, Int32 numBytes )
{
	return memcmp( s1, s2, numBytes );
}

#endif // !OS_PALM

