/*
 * _palm.h - General Palm layers.
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#ifndef OS_PALM

/*
 * Error Classes for each manager.
 */

#define errNone				0x0000	// No error

#define memErrorClass			0x0100	// Memory Manager
#define dmErrorClass			0x0200	// Data Manager
#define serErrorClass			0x0300	// Serial Manager
#define slkErrorClass			0x0400	// Serial Link Manager
#define sysErrorClass			0x0500	// System Manager
#define fplErrorClass			0x0600	// Floating Point Library
#define flpErrorClass			0x0680	// New Floating Point Library
#define evtErrorClass			0x0700	// System Event Manager
#define sndErrorClass			0x0800	// Sound Manager
#define almErrorClass			0x0900	// Alarm Manager
#define timErrorClass			0x0A00	// Time Manager
#define penErrorClass			0x0B00	// Pen Manager
#define ftrErrorClass			0x0C00	// Feature Manager
#define cmpErrorClass			0x0D00	// Connection Manager (HotSync)
#define dlkErrorClass			0x0E00	// Desktop Link Manager
#define padErrorClass			0x0F00	// PAD Manager
#define grfErrorClass			0x1000	// Graffiti Manager
#define mdmErrorClass			0x1100	// Modem Manager
#define netErrorClass			0x1200	// Net Library
#define htalErrorClass			0x1300	// HTAL Library
#define inetErrorClass			0x1400	// INet Library
#define exgErrorClass			0x1500	// Exg Manager
#define fileErrorClass			0x1600	// File Stream Manager
#define rfutErrorClass			0x1700	// RFUT Library
#define txtErrorClass			0x1800	// Text Manager
#define tsmErrorClass			0x1900	// Text Services Library
#define webErrorClass			0x1A00	// Web Library
#define secErrorClass			0x1B00	// Security Library
#define emuErrorClass			0x1C00	// Emulator Control Manager
#define flshErrorClass			0x1D00	// Flash Manager
#define pwrErrorClass			0x1E00	// Power Manager
#define cncErrorClass			0x1F00	// Connection Manager (Serial Communication)
#define actvErrorClass			0x2000	// Activation application
#define radioErrorClass			0x2100	// Radio Manager (Library)
#define dispErrorClass			0x2200	// Display Driver Errors.
#define bltErrorClass			0x2300	// Blitter Driver Errors.
#define winErrorClass			0x2400	// Window manager.
#define omErrorClass			0x2500	// Overlay Manager
#define menuErrorClass			0x2600	// Menu Manager

#define oemErrorClass			0x7000	// OEM/Licensee errors (0x7000-0x7EFF shared among ALL partners)
#define	 errInfoClass			0x7F00	// special class shows information w/o error code
#define appErrorClass			0x8000	// Application-defined errors

/*
 * StringMgr.
 */

inline int
TxtGlueCharIsAlNum( Char ch )
{
	return isalnum( ch );
}


inline Char*
StrCopy( Char* _dst, Char const* src )
{
	Assert( _dst );
	Assert( src );

	Char* dst = _dst;

	while( *src )
	{
	    *dst = *src;
	    dst++;
	    src++;
	}

	*dst = 0;

	return _dst;
}

inline Int16
StrLen( Char const* src )
{
	Int16 cch = 0;

	while( *src )
	{
	    cch++;
	    src++;
	}

	return cch;
}

#ifdef OS_WINDOWSMOBILE

int StrCompare( Char const* s1, Char const* s2 );
int StrCaselessCompare( Char const* s1, Char const* s2 );
int StrCaselessCompareCount( Char const* s1, Char const* s2, size_t cch );
int StrCaselessCompareSort( Char const* s1, Char const* s2 );

#else

inline int
StrCompare( Char const* s1, Char const* s2 )
{
	//$ WARNING: (chrisant) This yields a different sort than on an actual
	// Palm based device or Windows Mobile device.
	//$ WARNING: (chrisant) The high word is important -- the low word can
	// be zero when the high word is not!  Comparing 'Big W' and 'Bi lo'
	// was returning 0 due to Int16 truncation, when the complete real
	// return value was 0x00470000.
#ifdef OS_WINDOWS
	return lstrcmpA( s1, s2 );
#else
	return strcmp( s1, s2 );
#endif
}

inline int
StrCaselessCompare( Char const* s1, Char const* s2 )
{
	//$ WARNING: (chrisant) This yields a different sort than on an actual
	// Palm based device or Windows Mobile device.
#ifdef OS_WINDOWS
	return lstrcmpiA( s1, s2 );
#else
	return _stricmp( s1, s2 );
#endif
}

inline int
StrCaselessCompareCount( Char const* s1, Char const* s2, size_t cch )
{
	//$ WARNING: (chrisant) This yields a different sort than on an actual
	// Palm based device or Windows Mobile device.
	return _strnicmp( s1, s2, cch );
}

inline int
StrCaselessCompareSort( Char const* s1, Char const* s2 )
{
	//$ WARNING: (chrisant) This yields a different sort than on an actual
	// Palm based device or Windows Mobile device.
#ifdef OS_WINDOWS
	int n = lstrcmpiA( s1, s2 );
	if( !n )
	    n = lstrcmpA( s1, s2 );
	return n;
#else
	int n = _stricmp( s1, s2 );
	if( !n )
	    n = strcmp( s1, s2 );
	return n;
#endif
}

#endif // !OS_WINDOWS

/*
 * SysUtils.
 */

typedef int _comparF( void*, void*, Int32 other );
typedef _comparF* CmpFuncPtr;

void SysInsertionSort( void* baseP, Int16 numOfElements, Int16 width, CmpFuncPtr const comparF, Int32 const other );

/*
 * Date/Time.
 */

inline UInt32
TimGetSeconds()
{
	return UInt32( __localtime() ) + c_UnixToPalm;
}

#endif // !OS_PALM

