/*
 * data1x.h - Data Structures from 1.x
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#pragma pack( push, 2 )

#define v1MAX_STORES		15
#define v1cchMaxStoreName	32
#define v1cchMaxCurrency	4

struct v1AppInfoType
{
	// Categories.
	AppInfoType		appinfo;

	// Stores.
	UInt16			cStores;
	UInt16			bitStores;				// Mask of used store bits.
	UInt16			rgbitStores[v1MAX_STORES];
	UInt16			rgcItemsInStore[v1MAX_STORES+1];	// [0] is "All Stores" (total) count.
	Char			rgszStoreNames[v1MAX_STORES][v1cchMaxStoreName+1]; // Inefficient but ever-so-much simpler to code.
};
typedef v1AppInfoType* v1AppInfoPtr;

#define v1prefOneTime		0x0001
#define v1prefUseDecimals	0x0002
#define v1prefAutoQuantity	0x0004
#define v1prefNoFixCounts2	0x0010
struct v1PreferenceType
{
	UInt16			irecTopVisible;
	UInt16			flags;
	UInt16			store;
	UInt16			view;
	UInt16			columns;
	UInt16			catCurrent;
	// new in v1.05
	UInt16			exportFlags;
	// new in v1.60
	UInt16			font;
	// new in v1.70
	Char			szCurrency[v1cchMaxCurrency];
};

enum
{
	v1viewAll,
	v1viewNeed,
	v1viewCoupon,
	v1viewMAX,
};

enum
{
	v1colNeed		= 0x01,
	v1colName		= 0x02,
	v1colQuantity		= 0x04,
	v1colCoupon		= 0x08,
	v1colAisle		= 0x10,
	v1colCategory		= 0x20,
	v1colPrice		= 0x40,
};

enum
{
	v1exportQuantity	= 0x01,
	v1exportPrice		= 0x02,
	v1exportAisle		= 0x04,
};

struct v1Store
{
	UInt16			bit;
	UInt16			cItems;
	PCSZ			pszName;
};

#define v1flagNeed		0x01
#define v1flagCoupon		0x02
#define v1flagPrivate		0x80
#define v1storeOnce		0x8000
struct v1Item
{
	UInt8			flags;
	UInt8			cchName;		// Length of item description, excluding null terminator.
	UInt16			store;			// 16 bits mapping to stores.
	UInt16			uQuantity;
	UInt32			ulPrice;
	UInt8			aisle;
	Char			szName[1];		// The item description.
};
typedef v1Item* v1ItemPtr;
//$ WARNING: (chrisant) we want the size, minus szName.
#define v1sizeofItem		( sizeof( v1Item ) - 1 )
CASSERT( v1sizeofItem == 11 );


// Application database management defines.
#define v1AppType		'iShp'
#define v1DBType		'Data'
#define v1DBName		"HandyShopperDB"
#define v1PrefsVersionNum	24			// Preferences format version.
#define v1PrefsVersionNum_160	23			// Preferences format version from v1.60.
#define v1PrefsVersionNum_105	22			// Preferences format version from v1.05.
#define v1PrefsVersionNum_104	21			// Preferences format version from v1.04.
#define v1noRecordSelected	( UInt16( -1 ) )	// Index for no current record.

#pragma pack( pop )

