/*
 * PalmLayer.cpp - Palm compatibility layer.
 */

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#include "precomp.h"

#include "palmlayer.h"

/*
 * Time helpers.
 */

#if defined( OS_WINDOWS )

#if 0
static void
UnixTimeToFileTime( time_t t, LPFILETIME pft )
{
	// Note that LONGLONG is a 64-bit value.

	LONGLONG ll;

	ll = Int32x32To64( t, 10000000 ) + 116444736000000000;
	pft->dwLowDateTime = DWORD( ll );
	pft->dwHighDateTime = DWORD( ll >> 32 );
}
#endif

static time_t
FileTimeToUnixTime( const FILETIME *pft )
{
	// Note that LONGLONG is a 64-bit value.

	LONGLONG ll;

	// The nasty casts ensure the compiler doesn't sign extend the
	// r-values.

	*( ( ULONGLONG* )( void* )&ll ) = pft->dwLowDateTime;
	*( ( ULONGLONG* )( void* )&ll ) |= ULONGLONG( pft->dwHighDateTime ) << 32;

	// Convert to Unix time_t.

	ll -= 116444736000000000;
	ll /= 10000000;

	// Make sure it's between the Unix epoch and armageddon.

	if( ll < 0 || ll > INT_MAX )
	    return -1;

	// Return the converted time.

	return time_t( ll );
}

#endif // OS_WINDOWS

time_t
__localtime()
{
#if defined( OS_WINDOWS )
	SYSTEMTIME systime;
	FILETIME ftLocal;

	GetLocalTime( &systime );

	if( SystemTimeToFileTime( &systime, &ftLocal ) )
	    return FileTimeToUnixTime( &ftLocal );

	return -1;
#else
	return time( 0 );
#endif
}

/*
 * Utility.
 */

Err
__MemHandleUnlock( MemHandle h )
{
	return h ? MemHandleUnlock( h ) : 0;
}

Err
__MemHandleFree( MemHandle h )
{
	return h ? MemHandleFree( h ) : 0;
}

Err
__MemPtrUnlock( MemPtr pv )
{
	return pv ? MemPtrUnlock( pv ) : 0;
}

Err
__MemChunkFree( MemPtr pv )
{
	return pv ? MemChunkFree( pv ) : 0;
}

MemHandle
__MemPtrRecoverHandle( MemPtr pv )
{
	return pv ? MemPtrRecoverHandle( pv ) : 0;
}

