/*
 * Precompiled headers.
 */

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

/*
 * Determine platform.
 *
 * - OS_CRT		= Compiling for generic C runtime; no Win32 functions.
 * - OS_WINDOWS		= Compiling for a Win32 platform.
 * - OS_WINDOWSMOBILE	= Compiling for a Windows Mobile platform.
 * - OS_CRLF		= Compiling for a platform that ends lines with \r\n.
 * - OS_LF		= Compiling for a platform that ends lines with \n.
 * - OS_CR		= Compiling for a platform that ends lines with \r.
 *
 * If OS_CRT is not defined, we'll try to deduce the platform (just Windows
 * Mobile or Win32, at the moment).  If you need to add a new platform, be
 * sure to define OS_CRLF/OS_LF/OS_CR appropriately.
 */

#undef OS_WINDOWS
#undef OS_WINDOWSMOBILE

#ifdef UNICODE
#ifndef _UNICODE
#define _UNICODE
#endif
#else
#undef _UNICODE
#endif

#include "bldopts.h"

#if defined( OS_CRT )
#	define OS_LF
#elif defined( _WIN32_WCE )
#	define OS_WINDOWSMOBILE
#	define OS_WINDOWS
#	define OS_CRLF
#	define USE_DOS_PATH_PARSING
#	define USE_MEMORY_TRACKING
#elif defined( WIN32 )
#	define OS_WINDOWS
#	define OS_CRLF
#	define USE_DOS_PATH_PARSING
#	define USE_MEMORY_TRACKING
#else
#	error Unknown platform.
#endif

#if defined( OS_CRLF ) + defined( OS_LF ) + defined( OS_CR ) > 1
#	error Multiple line ending conventions defined; must define only one.
#elif defined( OS_CRLF ) + defined( OS_LF ) + defined( OS_CR ) < 1
#	error Line ending convention not defined; must define one.
#endif

#define INLINE_HRESULT_FROM_WIN32

#if defined( _MSC_VER )
#ifdef DEBUG
#pragma warning( disable: 4127 )
#endif
#pragma warning( disable: 4512 )
#endif

#define _USE_32BIT_TIME_T	// VS2005 defaults to 64 bit time_t on 32 bit platforms, but we expect 32 bit time_t.
#include <tchar.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#if defined( OS_WINDOWS )
#include <windows.h>
#include <windowsx.h>
#else
#include "windowsglue.h"
#endif

#if !defined( __STDC_WANT_SECURE_LIB__ )	// If this C++ compiler doesn't support the secure signatures, then set up some insecure shims so the code will at least compile.
#include "secureglue.h"
#endif

#if defined( OS_WINDOWSMOBILE )
#include <aygshell.h>
#include <commctrl.h>
#include <shlwapi.h>
#include <projects.h>
#elif defined( OS_WINDOWS )
#include <commctrl.h>
#endif

#include "stdmem.h"

#include "dbg.h"
#include "main.h"
#include "std.h"

