/*
 * Std.cpp - Standard definitions.
 */

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#include "precomp.h"

#include "std.h"
#include "str.h"

// Global variables.

bool g_fEmulator = false;

#ifdef DEBUG
bool g_fSlowPaints = false;
#endif

/*
 * LocaleSettings
 */

#ifdef NEED_LOCALE

LocaleSettings g_intl;

void
LocaleSettings::Refresh( bool const fZapPrefs )
{
	TCHAR sz[32];

#ifdef OS_WINDOWS
	sz[0] = 0;
	GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_ICOUNTRY, RgC( sz ) );
	int const iCountry = ( sz[0] ) ? _ttoi( sz ) : CTRY_UNITED_STATES;
	m_fCanada = ( iCountry == CTRY_CANADA );
#elif defined( OS_CRT )
	m_fCanada = false;
#else
#error LocaleSettings::Refresh not fully implemented for this platform.
#endif

	m_szDefaultCurrency[0] = 0;
#ifdef OS_WINDOWS
	GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_SCURRENCY, RgC( m_szDefaultCurrency ) );
#endif
	if( !m_szDefaultCurrency[0] )
	    CchCopy( TEXT( "$" ), RgC( m_szDefaultCurrency ) );
	if( fZapPrefs )
	    ZeroMemory( m_szCurrency, sizeof( m_szCurrency ) );

	sz[0] = 0;
#ifdef OS_WINDOWS
	GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_SMONDECIMALSEP, RgC( sz ) );
#endif
	m_chDecimal = sz[0] ? sz[0] : '.';

	sz[0] = 0;
#ifdef OS_WINDOWS
	GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_SMONTHOUSANDSEP, RgC( sz ) );
#endif
	m_chThousands = sz[0] ? sz[0] : ',';

	m_szDateSeparator[0] = 0;
#ifdef OS_WINDOWS
	GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_SDATE, RgC( m_szDateSeparator ) );
#endif
	if( !m_szDateSeparator[0] )
	    CchCopy( TEXT( "/" ), RgC( m_szDateSeparator ) );

#ifdef OS_WINDOWS
	static TCHAR const* const c_rgpszFormats[DimensionOf( m_rgszDateFormat )][3] =
	{
	    //	mdy			dmy				ymd
	    { TEXT( "M%sd" ),		TEXT( "d%sM" ),			TEXT( "M%sd" ) },
	    { TEXT( "M%sd%syy" ),	TEXT( "d%sM%syy" ),		TEXT( "yy%sM%sd" ) },
	    { TEXT( "ddd M%sd%syyyy" ),	TEXT( "ddd d%sM%syyyy" ),	TEXT( "ddd yyyy%sM%sd" ) },
	};
	sz[0] = 0;
	GetLocaleInfo( LOCALE_USER_DEFAULT, LOCALE_IDATE, RgC( sz ) );
	unsigned iOrder = 0;
	switch( sz[0] )
	{
	case '2':	iOrder = 2; break;
	case '1':	iOrder = 1; break;
	}
	for( unsigned ii = DimensionOf( m_rgszDateFormat ); ii--; )
	    wsprintf( m_rgszDateFormat[ii], c_rgpszFormats[ii][iOrder], GetDateSeparator(), GetDateSeparator() );
#endif
}

void
LocaleSettings::SetCurrency( TCHAR const* const psz )
{
	StrT s;

	s.Set( psz );

	if( s.Length() >= DimensionOf( m_szCurrency ) )
	    s.SetLength( DimensionOf( m_szCurrency ) - 1 );

	CchCopy( s.Text(), RgC( m_szCurrency ) );
}

void
LocaleSettings::SetCommaDecimalMode( bool const fUseCommaDecimal )
{
	if( fUseCommaDecimal )
	{
	    m_chThousands = '.';
	    m_chDecimal = ',';
	}
	else
	{
	    m_chThousands = ',';
	    m_chDecimal = '.';
	}
}

#ifdef OS_WINDOWS
TCHAR const*
LocaleSettings::GetDateFormatString( LocaleDateStyle const style ) const
{
	Assert( style >= 0 && style < DimensionOf( m_rgszDateFormat ) );
	return m_rgszDateFormat[style];
}

void
LocaleSettings::FormatDate( SYSTEMTIME const* const psystime, TCHAR* const pszOut, size_t const cchMax, LocaleDateStyle const style ) const
{
	if( cchMax )
	    *pszOut = 0;
	GetDateFormat( LOCALE_USER_DEFAULT, 0, psystime, GetDateFormatString( style ), pszOut, cchMax );
}
#endif

#endif // NEED_LOCALE

/*
 * Object.
 */

#ifdef DEBUG
void* _cdecl
Object::operator new( size_t cbSize )
{
	void* const pv = MemAlloc( cbSize _MEM_OBJECT );
	if( pv )
	    memset( pv, 0, cbSize );
	return pv;
}
#endif

