/*
 * Std.h - Standard definitions.
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#include "dbg.h"

#ifndef _WIN32_WCE
#include <limits.h>
#endif

/*
 * Defines.
 */

#if defined( _MSC_VER )
#if _MSC_VER >= 1400
#pragma warning( disable: 4481 )
#endif
#endif

#define syntax_safe			do {} while ( 0 )
#define syntaxsafe			syntax_safe

#define FImplies( x, y )		( !( x ) || ( y ) )
#define DimensionOf( x )		( sizeof( x ) / sizeof( ( x )[0] ) )
#define RgC( x )			( x ), DimensionOf( x )
#define OffsetOf( struc, member )	( DWORD( &( struc ).member ) - DWORD( &( struc ) ) )
#define OffsetOf0( struc, member )	( DWORD( &( ( ( struc* )0 )->member ) ) )
#define SameSize( x, y )		( sizeof( x ) == sizeof( y ) )

/*
 * Error handling.
 */

#ifdef OS_WINDOWS

inline HRESULT HrFromWin32( ULONG l ) { return HRESULT_FROM_WIN32( l ); }
inline long Win32FromHr( HRESULT hr ) { Assert( HRESULT_FACILITY( hr ) == FACILITY_WIN32 ); return ( HRESULT_FACILITY( hr ) == FACILITY_WIN32 ) ? ( long )( short int )( hr & 0xffff ) : 0; }

#undef HRESULT_FROM_WIN32
#define HRESULT_FROM_WIN32 __Please_Use_HrFromWin32__

#endif

#define CORg( x ) do { hr = ( x ); if( FAILED( hr ) ) goto LError; } while( 0 )

/*
 * Global variables.
 */

extern bool g_fEmulator;

#ifdef DEBUG
extern bool g_fSlowPaints;
#endif

/*
 * General helper templates.
 */

template <class T> T Min(T a, T b) { return (a < b) ? a : b; }
template <class T> T Max(T a, T b) { return (a > b) ? a : b; }
template <class T> T Abs(T x) { return (x < 0) ? -x : x; }

/*
 * LocaleSettings.
 */

#ifdef NEED_LOCALE

#ifdef OS_WINDOWS
enum LocaleDateStyle
{
	LDS_SHORT_NOYEAR,		// ex: 7/15
	LDS_SHORT,			// ex: 7/15/08
	LDS_MEDIUM,			// ex: Tue 7/15/2008
	LDS_MAX
};
#endif

class LocaleSettings
{
    public:
			LocaleSettings() { Refresh( true ); }

	void		Refresh( bool fZapPrefs = false );

	bool		IsCanada() const { return m_fCanada; }
	TCHAR const*	GetCurrency() const { return *m_szCurrency ? m_szCurrency : m_szDefaultCurrency; }
	TCHAR const*	GetDefaultCurrency() const { return m_szDefaultCurrency; }
	TCHAR		GetThousandsSeparator() const { return m_chThousands; }
	TCHAR		GetDecimalSeparator() const { return m_chDecimal; }
	TCHAR const*	GetDateSeparator() const { return m_szDateSeparator; }

#ifdef OS_WINDOWS
	void		FormatDate( SYSTEMTIME const* psystime, TCHAR* pszOut, size_t cchMax, LocaleDateStyle style ) const;
#endif

	void		SetCurrency( TCHAR const* psz );
	void		SetCommaDecimalMode( bool fUseCommaDecimal );

    protected:
#ifdef OS_WINDOWS
	TCHAR const*	GetDateFormatString( LocaleDateStyle style ) const;
#endif

    private:
	bool		m_fCanada;
	TCHAR		m_szCurrency[8];
	TCHAR		m_szDefaultCurrency[8];
	TCHAR		m_chThousands;
	TCHAR		m_chDecimal;
	TCHAR		m_szDateSeparator[4];
#ifdef OS_WINDOWS
	TCHAR		m_rgszDateFormat[LDS_MAX][32];
#endif
};

extern LocaleSettings g_intl;

#endif // NEED_LOCALE

/*
 * Override helpers for virtual methods.
 */

#define OVERRIDE
#if defined( _MSC_VER )
#if _MSC_VER >= 1400
#undef OVERRIDE
#define OVERRIDE override
#endif
#endif

/*
 * Use in place of virtual for overridden member functions.
 *
 * Where you would have used:
 *
 *	virtual Type Func(...arg list...);
 *
 *	virtual Type Func(...arg list...) const;
 *
 *	virtual STDMETHOD Func(...arg list...);
 *
 *	virtual STDMETHOD_(type, Func)(...arg list...) const;
 *
 * Now use:
 *
 *	Override(Base, Type, Func, (...arg list...));
 *
 *	OverrideConst(Base, Type, Func, (...arg list...));
 *
 *	OverrideStdMethod(Base, Func, (...arg list...));
 *
 *	OverrideStdMethod_(Base, Type, Func, (...arg list...));
 *
 * If the base function changes signature, you will now get a compiler warning
 * in the derived class.
 *
 * Use the OverrideStdMethod() to override interface functions that use
 * STDMETHOD.  WARNING:  the macro is dependent on STDMETHOD being defined as
 * using _stdcall.
*/

// Use this template or macro to fool the compiler into suppressing the
// 'local variable is initialized but not referenced' warning (C4189).
#if defined( _MSC_VER )
#pragma warning( push )
#pragma warning( disable: 4100 )
#endif
// for C++ code
template <class T> inline void SUPPRESS_UNUSED_VAR( T var ) {}
template <class T1, class T2> inline void SUPPRESS_UNUSED_VAR2( T1 var1, T2 var2 ) {}
template <class T1, class T2, class T3> inline void SUPPRESS_UNUSED_VAR3( T1 var1, T2 var2, T3 var3 ) {}
template <class T1, class T2, class T3, class T4> inline void SUPPRESS_UNUSED_VAR4( T1 var1, T2 var2, T3 var3, T4 var4 ) {}
// for C code
#define SUPPRESS_UNUSED_VAR_C( var ) do { if ( DWORD_PTR( LPVOID( &( var ) ) ) & 1 ) {} } while ( 0 )
#if defined( _MSC_VER )
#pragma warning( pop )
#endif

#if defined( _MSC_VER )

// For the Microsoft C++ compiler we can decorate the method declaration to
// get the compiler to verify that the method really is overriding an existing
// virtual.  The best support for this came in VS2005 which introduced the
// native "override" keyword.

#if _MSC_VER >= 1400

#define OVERRIDE override

#define Override( Base, Type, Func, Args ) \
	virtual Type Func##Args override

#define OverrideConst( Base, Type, Func, Args ) \
	virtual Type Func##Args const override

#define OverrideStdMethod( Base, Func, Args ) \
	STDMETHOD( Func )##Args override

#define OverrideStdMethod_( Base, Type, Func, Args ) \
	STDMETHOD_( Type,Func )##Args override

#elif _MSC_VER >= 1310

#define Override( Base, Type, Func, Args ) \
	void Override##Func() { \
	    int signature_of_##Func##_is_wrong( Type ( Base::* ) Args ); \
	    struct X : public Base { \
		enum { Dummy = sizeof( signature_of_##Func##_is_wrong( &X::Func ) ) }; \
	    }; } \
	virtual Type Func##Args

#define OverrideConst( Base, Type, Func, Args ) \
	void Override##Func() { \
	    int signature_of_##Func##_is_wrong( Type ( Base::* ) Args const ); \
	    struct X : public Base { \
		enum { Dummy = sizeof( signature_of_##Func##_is_wrong( &X::Func ) ) }; \
	    }; } \
	virtual Type Func##Args const

//$ this is HIGHLY dependent on STDMETHOD being defined as using _stdcall.
#define OverrideStdMethod( Base, Func, Args ) \
	void Override##Func() { \
	    int signature_of_##Func##_is_wrong( HRESULT ( _stdcall Base::* ) Args ); \
	    struct X : public Base { \
		enum { Dummy = sizeof( signature_of_##Func##_is_wrong( &X::Func ) ) }; \
	    }; } \
	STDMETHOD( Func )##Args

#define OverrideStdMethod_( Base, Type, Func, Args ) \
	void Override##Func() { \
	    int signature_of_##Func##_is_wrong( Type ( _stdcall Base::* ) Args ); \
	    struct X : public Base { \
		enum { Dummy = sizeof( signature_of_##Func##_is_wrong( &X::Func ) ) }; \
	    }; } \
	STDMETHOD_( Type,Func )##Args

#else	// _MSC_VER < 1310

#define Override( Base, Type, Func, Args ) \
	void Override##Func() {Type ( Base:: *pfn )##Args = Base::Func; SUPPRESS_UNUSED_VAR( pfn );} \
	virtual Type Func##Args

#define OverrideConst( Base, Type, Func, Args ) \
	void Override##Func() {Type ( Base:: *pfn )##Args; \
			*( Type ( Base::** )##Args const ) &pfn = Base::Func; \
			SUPPRESS_UNUSED_VAR( pfn ); } \
	virtual Type Func##Args const

//$ this is HIGHLY dependent on STDMETHOD being defined as using _stdcall.
#define OverrideStdMethod( Base, Func, Args ) \
	void Override##Func() {HRESULT ( _stdcall Base:: *pfn )##Args; \
			*( HRESULT ( _stdcall Base::** )##Args ) &pfn = Base::Func; \
			SUPPRESS_UNUSED_VAR( pfn ); } \
	STDMETHOD( Func )##Args

#define OverrideStdMethod_( Base, Type, Func, Args ) \
	void Override##Func() {Type ( _stdcall Base:: *pfn )##Args; \
			*( Type ( _stdcall Base::** )##Args ) &pfn = Base::Func; \
			SUPPRESS_UNUSED_VAR( pfn ); } \
	STDMETHOD_( Type,Func )##Args

#endif	// _MSC_VER < 1310

#else

// For unrecognized compilers don't use any markup for overrides.

#define Override( Base, Type, Func, Args ) \
	virtual Type Func##Args

#define OverrideConst( Base, Type, Func, Args ) \
	virtual Type Func##Args const

#define OverrideStdMethod( Base, Func, Args ) \
	STDMETHOD( Func )##Args

#define OverrideStdMethod_( Base, Type, Func, Args ) \
	STDMETHOD_( Type,Func )##Args

#endif

/*
 * Smart handles.
 */

#include "smartpointers.h"

#if defined( _MSC_VER )
#if _MSC_VER >= 1400
#pragma warning( disable: 4284 )
#endif
#endif

template <class Type, Type EmptyValue, class Subclass>
class SH : public Subclass
{
    public:
	SH( Type h = EmptyValue )	{ m_h = h; }
	~SH()				{ if( EmptyValue != m_h ) Subclass::Free( m_h ); }
	operator Type() const		{ return m_h; }
	Type Handle() const		{ return m_h; }
	Type* operator&()		{ Assert1( EmptyValue == m_h, "Non-empty %s as out param.", TypeIdName3( SH<Type, EmptyValue, Subclass> ) ); return &m_h; }
	Type operator=( Type h )	{ Assert1( EmptyValue == m_h, "Non-empty %s in assignment.", TypeIdName3( SH<Type, EmptyValue, Subclass> ) ); return m_h = h; }
	Type Transfer()			{ Type const h = m_h; m_h = EmptyValue; return h; }
	void Free()			{ if( EmptyValue != m_h ) Subclass::Free( m_h ); m_h = EmptyValue; }
	void Close()			{ Free(); }
	bool operator!() const		{ CASSERT( EmptyValue == 0 ); return EmptyValue == m_h; }
	bool IsEmpty() const		{ return EmptyValue == m_h; }

	Type* UnsafeAddress()		{ return &m_h; }

    protected:
	Type m_h;

    private:
	SH<Type, EmptyValue, Subclass>& operator=( SH<Type, EmptyValue, Subclass> const& sh ); // Forbidden.
	SH( SH<Type, EmptyValue, Subclass> const& ); // Forbidden.
};

template <class Type, class Subclass>
class SP : public Subclass
{
    public:
	SP( Type* p = 0 )		{ m_p = p; }
	~SP()				{ if( m_p ) Subclass::Free( m_p ); }
	operator Type*() const		{ return m_p; }
	Type* Pointer() const		{ return m_p; }
	Type& operator*() const		{ Assert( m_p ); return *m_p; }
	Type& operator[]( int i ) const	{ Assert( m_p ); return m_p[i]; }
	Type& operator[]( unsigned i ) const { Assert( m_p ); return m_p[i]; }
	Type* operator&()		{ Assert1( !m_p, "Non-empty %s as out param.", TypeIdName2( SP<Type, Subclass> ) ); return &m_p; }
	Type* operator=( Type* p )	{ Assert1( !m_p, "Non-empty %s in assignment.", TypeIdName2( SP<Type, Subclass> ) ); return m_p = p; }
	Type* Transfer()		{ Type* const p = m_p; m_p = 0; return p; }
	void Free()			{ if( m_p ) Subclass::Free( m_p ); m_p = 0; }
	void Close()			{ Free(); }
	Type* operator->() const	{ return m_p; }
	bool operator!() const		{ return !m_p; }
	bool IsEmpty() const		{ return !m_p; }

	Type** UnsafeAddress()		{ return &m_p; }

    protected:
	Type* m_p;

    private:
	SP<Type, Subclass>& operator=( SP<Type, Subclass> const& sh ); // Forbidden.
	SP( SP<Type, Subclass> const& ); // Forbidden.
};

#if defined( OS_WINDOWS )

class SH_CloseHandle
{
    protected:
	void Free( HANDLE h ) { CloseHandle( h ); }
};

class SH_FindClose
{
    protected:
	void Free( HANDLE h ) { FindClose( h ); }
};

class SH_RegCloseKey
{
    protected:
	void Free( HKEY hkey ) { RegCloseKey( hkey ); }
};

typedef SH<HANDLE, NULL, SH_CloseHandle> SHandle;
typedef SH<HANDLE, INVALID_HANDLE_VALUE, SH_CloseHandle> SFileHandle;
typedef SH<HANDLE, INVALID_HANDLE_VALUE, SH_FindClose> SFindHandle;
typedef SH<HKEY, NULL, SH_RegCloseKey> SHKEY;

#elif defined( OS_CRT )

class SP_FILE
{
    protected:
	void Free( FILE* file ) { fclose( file ); }
};

typedef SP<FILE, SP_FILE> SPFILE;

#endif

/*
 * Object - base class with virtual destructor.
 *
 * Also the virtual destructor allows RTTI (run-time type information) to show
 * the friendly name of the most derived class.
 */

class Object
{
    public:
	virtual		~Object() {}

#ifdef DEBUG
    public:
	void* _cdecl operator new( size_t cbSize );
#endif // DEBUG
};

/*
 * PreserveLastError - scoped save/restore for GetLastError() or errno.
 */

class PreserveLastError
{
    public:
	PreserveLastError();
	~PreserveLastError();
    private:
#if defined( OS_WINDOWS )
	DWORD m_dwLastError;
#elif defined( OS_CRT )
	errno_t m_errno;
#else
#error PreserveLastError is not defined for this platform.
#endif
};

inline PreserveLastError::PreserveLastError()
{
#if defined( OS_WINDOWS )
	m_dwLastError = GetLastError();
#elif defined( OS_CRT )
	m_errno = errno;
#endif
}

inline PreserveLastError::~PreserveLastError()
{
#if defined( OS_WINDOWS )
	SetLastError( m_dwLastError );
#elif defined( OS_CRT )
	errno = m_errno;
#endif
}

