/*
 * Str.cpp - Simplistic string class.
 */

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#include "precomp.h"

#include <stdio.h>

#include "str.h"

#ifdef UNICODE

#if defined( OS_CRT )

size_t
WideToNarrow( WCHAR const* pwzFrom, size_t cchFrom, char* pszTo, size_t cchMax )
{
	Assert( size_t( -1 ) != cchFrom );

	if( pszTo )
	{
	    char* const pszBegin = pszTo;

	    while( cchFrom-- && cchMax-- )
	    {
		if( *pwzFrom >= 1 && *pwzFrom <= 127 )
		    *pszTo = LOBYTE( *pwzFrom );
		else
		    *pszTo = '?';
	    }

	    return pszTo - pszBegin;
	}
	else
	{
	    return cchFrom / sizeof( *pwzFrom );
	}
}

size_t
NarrowToWide( char const* pszFrom, size_t cchFrom, WCHAR* pwzTo, size_t cchMax )
{
	Assert( size_t( -1 ) != cchFrom );

	if( pwzTo )
	{
	    WCHAR* const pwzBegin = pwzTo;

	    while( cchFrom-- && cchMax-- )
	    {
		if( *pszFrom >= 1 && *pszFrom <= 127 )
		    *pwzTo = *pszFrom;
		else
		    *pwzTo = '?';
	    }

	    return pwzTo - pwzBegin;
	}
	else
	{
	    return cchFrom / sizeof( *pszFrom );
	}
}

#endif

size_t
WideToNarrowString( WCHAR const* pwzFrom, char* pszTo, size_t cchMax )
{
	size_t const cchFrom = CchLen( pwzFrom );
	size_t const cch = WideToNarrow( pwzFrom, cchFrom, pszTo, cchMax );

	if( pszTo )
	    pszTo[cch] = 0;

	return cch;
}

size_t
NarrowToWideString( char const* pszFrom, WCHAR* pwzTo, size_t cchMax )
{
	size_t const cchFrom = CchLen( pszFrom );
	size_t const cch = NarrowToWide( pszFrom, cchFrom, pwzTo, cchMax );

	if( pwzTo )
	    pwzTo[cch] = 0;

	return cch;
}

#else // !UNICODE

size_t
CharToCharN( char const* pszFrom, size_t cchFrom, char* pszTo, size_t cchMax )
{
	Assert( size_t( -1 ) != cchFrom );

	if( pszTo )
	{
	    char* const pszBegin = pszTo;

	    while( cchFrom-- && cchMax-- )
		*pszTo = *pszFrom;

	    return pszTo - pszBegin;
	}
	else
	{
	    // Return how much space is needed, in characters.
	    return cchFrom / sizeof( *pszFrom );
	}
}

#endif // !UNICODE

#ifdef UNICODE

void
StrA::SetW( WCHAR const* psz, size_t cch )
{
	Clear();

	if( cch == size_t( -1 ) )
	    cch = CchLen( psz );

	size_t const cchNarrow = WideToNarrow( psz, cch, 0, 0 );

	AllocInternal( cchNarrow );

	m_cchLen = WideToNarrow( psz, cch, m_psz, cchNarrow );
	m_psz[m_cchLen] = 0;
}

void
StrW::SetA( char const* psz, size_t cch )
{
	Clear();

	if( cch == size_t( -1 ) )
	    cch = CchLen( psz );

	size_t const cchWide = NarrowToWide( psz, cch, 0, 0 );

	AllocInternal( cchWide );

	m_cchLen = NarrowToWide( psz, cch, m_psz, cchWide );
	m_psz[m_cchLen] = 0;
}

#endif // UNICODE

