/*
 * Sys.h - System abstractions.
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

/*
 * SysError - Simple general error codes.
 */

enum SysError
{
	SE_OK,

	// General error codes.

	SE_NOTIMPLEMENTED,
	SE_OUTOFMEMORY,
	SE_FILENOTFOUND,
	SE_ACCESSDENIED,
	SE_FILEEXISTS,
	SE_DISKFULL,
	SE_FILEINUSE,
	SE_UNKNOWN,

	// App-specific error codes.

	SE_INTERNALERROR,
	SE_UNRECOGNIZEDDATABASE,
	SE_OLDFORMATDATABASE,
	SE_BUFFERTOOSMALL,
	SE_DATABASENOTOPEN,
	SE_CORRUPT,

	// Total number of errors.

	SE_MAX
};

CASSERT( SE_OK == 0 );

/*
 * Filename helpers.
 */

#if defined( USE_DOS_PATH_PARSING )
TCHAR const c_chPreferredDirectorySeparator = '\\';
inline bool IsDirectorySeparator( TCHAR ch ) { return ch == '\\' || ch == '/'; }
#else
#error Path parsing not implemented for this platform.
#endif

bool FileExists( TCHAR const* pszFile );
TCHAR const* GetFileName( TCHAR const* pszFile );
TCHAR const* GetExtension( TCHAR const* pszFile );
TCHAR const* StripFilename( TCHAR const* pszPath, StrT& sPathOnly, StrT* psFileOnly = 0 );
SysError __DeleteFile( TCHAR const* pszFile );
SysError __RenameFile( TCHAR const* pszFrom, TCHAR const* pszTo );

/*
 * FileIO - File IO abstraction.
 */

enum FileMode
{
	FM_READ,
	FM_READWRITE,
	FM_WRITE,
	FM_MAX
};

class FileIO
{
    public:
	static FileIO*	Create();

			~FileIO();

	void		Set( TCHAR const* pszFile );
	TCHAR const*	Name() const { return m_sName.Text(); }
	bool		Open( FileMode mode, SysError& e );
	bool		Seek( unsigned offset, SysError& e );
	bool		Read( void* pvBuffer, unsigned cbToRead, SysError& e, unsigned* pcbDidRead = 0 );
	bool		Write( void const* pvBuffer, unsigned cbToWrite, SysError& e );
	unsigned	Offset( SysError& e );
	unsigned	Size( SysError& e );
	bool		Truncate( SysError& e );
	void		Close();
	bool		Delete( SysError& e );
	bool		Rename( TCHAR const* pszNewName, SysError& e );

    protected:
			FileIO() : m_mode( FM_READ ) {}

	bool		IsSet() const { return !!m_sName.Length(); }
	bool		IsOpen() const { return !m_shFile.IsEmpty(); }

	StrT		m_sName;
	FileMode	m_mode;

#if defined( OS_WINDOWS )
	SFileHandle	m_shFile;
#elif defined( OS_CRT )
	SPFILE		m_shFile;
#else
#error FileIO not implemented for this platform.
#endif
};

