/*
 * WindowsGlue.h - Ease compilation for non-Windows platforms.
 */

#pragma once

// vim: set noet:			// Hard tabs.
// vim: set ts=8:			// Tab width 8.
// vim: set sw=4:			// Shift (indent) width 4.
// vim: set cino={0s:			// Braces flush with outer statement.
// vim: set cino=^1s:			// First indent 8 (add 1 shift width).

#include <ctype.h>

/*
 * CASSERT is a compile-time assertion, and can test any expression that can
 * be computed at compile time.  If the expression evaluates to false, it
 * generates a compile error.
 */

#define CASSERT( expression ) typedef int dummy_array__[( expression ) ? 1 : -1]

/*
 * Define some Windows types.
 */

typedef unsigned char BYTE;
typedef BYTE* LPBYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
typedef long LONG;
typedef unsigned long ULONG;

#ifdef UNICODE
typedef wchar_t WCHAR;
#endif

#ifdef UNICODE
typedef WCHAR TCHAR;
#else
typedef char TCHAR;
#endif

typedef char const* LPCSTR;
typedef TCHAR const* LPCTSTR;
typedef char* LPSTR;
typedef TCHAR* LPTSTR;

#ifdef UNICODE
typedef WCHAR const* LPCWSTR;
typedef WCHAR* LPWSTR;
#endif

#define TEXT( x ) _TEXT( x )

typedef unsigned long ULONG_PTR;
typedef unsigned long DWORD_PTR;
CASSERT( sizeof( ULONG_PTR ) == sizeof( void* ) );

struct SYSTEMTIME
{
	WORD wYear;
	WORD wMonth;
	WORD wDayOfWeek;
	WORD wDay;
	WORD wHour;
	WORD wMinute;
	WORD wSecond;
	WORD wMilliseconds;
};
typedef SYSTEMTIME* PSYSTEMTIME;
typedef SYSTEMTIME* LPSYSTEMTIME;

typedef DWORD COLORREF;

/*
 * Define analogs to some Windows functions.
 */

inline void ZeroMemory( void* pv, size_t cb ) { memset( pv, 0, cb ); }

inline bool IsCharAlphaNumericA( char ch ) { return !!isalnum( ch ) ; }
#ifdef UNICODE
inline bool IsCharAlphaNumericW( WCHAR ch ) { return !!iswalnum( ch ) ; }
#endif

#define LOBYTE( x ) ( static_cast<unsigned char>( ( x ) & 0xff ) )
#define LOWORD( x ) ( static_cast<unsigned short>( ( x ) & 0xffff ) )

/*
 * COLORREF is an RGB value encoded as a DWORD.
 */

#define GetRValue( rgb )	( LOBYTE( rgb ) )
#define GetGValue( rgb )	( LOBYTE( ( WORD( rgb ) ) >> 8 ) )
#define GetBValue( rgb )	( LOBYTE( ( rgb ) >> 16 ) )
#define RGB( r, g, b )		( COLORREF( ( BYTE( r ) | ( WORD( BYTE( g ) ) << 8 ) ) | ( ( DWORD ( BYTE( b ) ) )<<16 ) ) )

